package Dist::Zilla::PluginBundle::BOKUTIN;
# ABSTRACT: Dist::Zilla plugins for me
$Dist::Zilla::PluginBundle::BOKUTIN::VERSION = '0.2015071600';
use Moose;
use Moose::Util::TypeConstraints;
use Dist::Zilla;
use Data::Section::Simple qw(get_data_section);
use List::MoreUtils qw(any);
with 'Dist::Zilla::Role::PluginBundle::Easy';

has dist => (
    is       => 'ro',
    isa      => 'Str',
    required => 0,
);

has authority => (
    is      => 'ro',
    isa     => 'Str',
    default => 'cpan:BOKUTIN',
);

has github_user => (
    is      => 'ro',
    isa     => 'Str',
    default => 'bokutin',
);

has github_name => (
    is      => 'ro',
    isa     => 'Str',
    lazy    => 1,
    default => sub { $_[0]->dist },
);

has repository => (
    is  => 'ro',
    isa => 'Str',
);

for my $attr (qw(repository_type repository_url repository_web)) {
    has $attr => (
        is      => 'ro',
        isa     => 'Maybe[Str]',
        lazy    => 1,
        default => sub {
            my $self = shift;
            my $data = $self->_repository_data;
            return unless $data;
            return $data->{$attr};
        },
    );
}

sub _repository_data {
    my $self = shift;

    my $host = $self->repository;
    return unless defined $host;

    die "Unknown repository host $host"
        unless exists $self->_repository_host_map->{$host};

    return $self->_repository_host_map->{$host};
}

has _repository_host_map => (
    is      => 'ro',
    isa     => 'HashRef[HashRef[Str]]',
    lazy    => 1,
    default => sub {
        my $self = shift;
        return {
            'github' => {
                repository_type => 'git',
                repository_url  => sprintf('git://github.com/%s/%s.git', $self->github_user, $self->github_name),
                repository_web  => sprintf('https://github.com/%s/%s', $self->github_user, $self->github_name),
            },
        }
    },
);

has bugtracker_web => (
    is      => 'ro',
    isa     => 'Str',
    lazy    => 1,
    default => sub {
        sprintf('http://rt.cpan.org/Public/Dist/Display.html?Name=%s',
                shift->dist);
    },
);

has bugtracker_mailto => (
    is      => 'ro',
    isa     => 'Str',
    lazy    => 1,
    default => sub { sprintf('bug-%s@rt.cpan.org', lc shift->dist); },
);

has homepage => (
    is      => 'ro',
    isa     => 'Str',
    lazy    => 1,
    default => sub { sprintf('http://metacpan.org/release/%s', shift->dist) },
);

has awesome => (
    is  => 'ro',
    isa => 'Str',
);

has dynamic_config => (
    is  => 'ro',
    isa => 'Str',
);

has is_task => (
    is      => 'ro',
    isa     => 'Bool',
    lazy    => 1,
    default => sub { shift->dist =~ /^Task-/ ? 1 : 0 },
);

has is_test_dist => (
    is      => 'ro',
    isa     => 'Bool',
    lazy    => 1,
    default => sub {
        my $self = shift;
        return 1 if $ENV{DZIL_FAKE_RELEASE};
        return $self->dist =~ /^Foo-/ ? 1 : 0
    },
);

has done_testing => (
    is      => 'ro',
    isa     => 'Bool',
    default => 1,
);

has _plugins => (
    is      => 'ro',
    isa     => 'ArrayRef[Str]',
    lazy    => 1,
    default => sub {
        my $self = shift;
        [
            qw(
                CPANFile
                GatherDir
                PruneCruft
                ManifestSkip
                MetaYAML
                License
                RunExtraTests
                ExecDir
                ShareDir
            ),
            ($self->awesome ? $self->awesome : 'MakeMaker'),
            (
                "Manifest",
                "TestRelease",
                "ConfirmRelease",
                "MetaConfig",
                "MetaJSON",
                "NextRelease",
                #"CheckChangesHasContent",
                "PkgVersion",
                "Authority",
                "PodCoverageTests",
                "PodSyntaxTests",
                "NoTabsTests",
                "EOLTests",
                "Test::Compile",
                "Metadata",
                "MetaResources",
                "Git::Commit",
                "Git::Tag",
                "Git::NextVersion",
            ),
            ($self->is_task      ? 'TaskWeaver'  : 'PodWeaver'),
            ($self->is_test_dist ? 'FakeRelease' : 'UploadToCPAN'),
        ]
    },
);

has plugin_options => (
    is       => 'ro',
    isa      => 'HashRef[HashRef[Str]]',
    init_arg => undef,
    lazy     => 1,
    default  => sub {
        my $self = shift;
        my %opts = (
            'NextRelease'        => { format => '%-5v %{yyyy-MM-dd}d' },
            'Authority'          => { authority => $self->authority },
            'Git::Tag'           => { tag_format => '%v', tag_message => '' },
            'Git::NextVersion' => {
                version_regexp => '^(\d+\.\d+)$',
                first_version  => '0.01'
            },
            'Git::Commit' => {
                allow_dirty => 1,
            },
        );

        $opts{Metadata} = {
            dynamic_config => 1,
        } if $self->dynamic_config;

        for my $metaresource (qw(repository.type repository.url repository.web bugtracker.web bugtracker.mailto homepage)) {
            (my $method = $metaresource) =~ s/\./_/g;
            my $value = $self->$method;
            if (!$value) {
                warn "*** resources.$metaresource is not configured! This needs to be fixed! ***"
                    unless $metaresource eq 'bugtracker.mailto';
                next;
            }
            $opts{MetaResources}{$metaresource} = $value;
        }
        delete $opts{MetaResources}{'repository.type'}
            unless exists $opts{MetaResources}{'repository.url'};

        for my $option (keys %{ $self->payload }) {
            next unless $option =~ /^([A-Z][^_]*)_(.+)$/;
            my ($plugin, $plugin_option) = ($1, $2);
            $opts{$plugin} ||= {};
            $opts{$plugin}->{$plugin_option} = $self->payload->{$option};
        }

        return \%opts;
    },
);

has publicity => (
    is  => 'ro',
    isa => subtype( 'Str' => where { m/^(darkpan|privpan)$/ } ),
    default => sub { 'darkpan' },
);

has exclude_match => (
    is  => 'ro',
    isa => 'ArrayRef',
    default => sub { [] },
);

around BUILDARGS => sub {
    my $orig = shift;
    my $class = shift;
    my $args = $class->$orig(@_);
    return { %{ $args->{payload} }, %{ $args } };
};

sub mvp_multivalue_args { qw(exclude_match) }

sub configure {
    my $self = shift;

    $self->add_plugins(
        [
            GatherDir => {
                #exclude_filename => [qw(Makefile.PL cpanfile .ukigumo.yml)],
                #exclude_filename => [],
                exclude_match    => [qw(^_ ^share/sausalito ^submod/), @{$self->exclude_match}],
                #include_dotfiles => 1,
            },
        ],
    );

    $self->add_bundle(
        '@Filter' => {
            -bundle => '@Basic',
            -remove => [qw(Readme GatherDir MakeMaker PruneCruft UploadToCPAN)],
        },
    );

    if (-d 'inc') {
        $self->add_plugins(
            [ '=inc::MyDistMakeMaker', 'MyDistMakeMaker' ],
        );
    }
    else {
        $self->add_plugins(
            'MakeMaker',
        );
    }

    #if ($self->publicity eq 'darkpan') {
    #    $self->add_plugins(
    #        'Git::Init',
    #        'GitHub::Create',
    #    );
    #}

    $self->add_plugins(
        'CPANFile',
        #'CheckChangesHasContent',
        #[
        #    GenerateFile => {
        #        filename => '.ukigumo.yml',
        #        content => get_data_section('.ukigumo.yml'),
        #    },
        #],
        'PkgVersion',
        [
            PruneCruft => {
                except => [qr/^\.ukigumo\.yml$/],
            },
        ],
        #[
        #    CopyFilesFromBuild => {
        #        copy => [qw(Makefile.PL cpanfile .ukigumo.yml)],
        #    },
        #],
        [
            AutoVersion => {
                major  => 0,
                #format => q<{{ $major }}.{{ cldr('yyDDD') }}{{ sprintf('%01u', ($ENV{N} || 0)) }}{{$ENV{DEV} ? (sprintf '_%03u', $ENV{DEV}) : ''}}>,
                format => q<{{ $major }}.{{ cldr('yyyyMMdd') }}{{ sprintf('%02u', ($ENV{N} || 0)) }}{{$ENV{DEV} ? (sprintf '_%02u', $ENV{DEV}) : ''}}>,
            },
        ],
        [
            NextRelease => {
                format => '%-9v %{yyyy-MM-dd eee HH:mm:ss VVVV}d',
            },
        ],
        'Git::Commit',
        'Git::Tag',
        [
            ScpDeploy => {
                hosts      => '192.168.11.42',
                remote_dir => "./@{[ $self->publicity ]}/authors/id/B/BO/BOKUTIN",
                command    => 'bin/darkpan_indexer',
            },
        ],
        'PodWeaver',
        #[
        #    'Upload::SCP' => {
        #        connection => 'bokut.in',
        #        directory  => 'public_html/darkpan/authors/id/B/BO/BOKUTIN',
        #        clobber    => 1,
        #    },
        #],
    );

    #$self->add_plugins(
    #    [ 'Prereqs' => 'TestMoreDoneTesting' => {
    #        -phase       => 'test',
    #        -type        => 'requires',
    #        'Test::More' => '0.88',
    #    } ]
    #) if $self->done_testing;
    #$self->add_plugins(
    #    map { [ $_ => ($self->plugin_options->{$_} || {}) ] }
    #        @{ $self->_plugins },
    #);
}

__PACKAGE__->meta->make_immutable;
no Moose;
1;

=pod

=head1 NAME

Dist::Zilla::PluginBundle::BOKUTIN - Dist::Zilla plugins for me

=head1 VERSION

version 0.2015071600

=head1 AUTHOR

Tomohiro Hosaka <bokutin@bokut.in>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by Tomohiro Hosaka.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut

__DATA__

@@ .ukigumo.yml
# 指定しないと、デフォルトで cpanm --notest --installdeps . が走る。
# ここでエラると Fail レポートが送られないため、t/00prereq.t で同じことをする。
# true を指定して、デフォルトで cpanm をしない。
install: true
before_install:
  #- "dzil authordeps --missing | grep -v '^inc' | cpanm -n"
  #- "dzil listdeps --missing | cpanm -n"
  #- "dzil build"
  - "git submodule foreach git pull origin master"
  #- "cd submod/BOKUTIN-Utils && dzil listdeps --missing | cpanm -n"
